#include <windows.h>
#include <wtypes.h>
#include "wmsdk.h"
#include <stdio.h>
#include "resource.h"
#include "plugin.h"


static HINSTANCE hInstance = NULL;
static POINT position = {0, 0};
static DWORD enabled = FALSE;


HRESULT (STDMETHODCALLTYPE *myWMCreateSyncReader)( IUnknown* pUnkCert, DWORD dwRights, IWMSyncReader **ppSyncReader );


BOOL APIENTRY DllMain( HANDLE hModule, DWORD reason, LPVOID resvd )
{
  hInstance = (HINSTANCE)hModule;
  return TRUE;
}


static void FormatString(char *in, char *fname, char *out, int len,
			 char *title, char *artist, char *album,
			 char *year, int track, char *bitrate, char *duration)
{
  int i, j, slen;
  char obuff[8192];
  char buff[8];
	
  if (in[0]==0) { strncpy(out, fname, len); return; }

  slen = strlen(in);
  for (i=j=0; i<slen && j<8000; i++) {
    obuff[j] = 0;
    if (!strncmp(in+i, "%title%", 7)) {
      strcat(obuff+j, title);
      i += 6;
      j += strlen(title);
    } else if (!strncmp(in+i, "%artist%", 8)) {
      strcat(obuff+j, artist);
      i += 7;
      j += strlen(artist);
    } else if (!strncmp(in+i, "%album%", 7)) {
      strcat(obuff+j, album);
      i += 6;
      j += strlen(album);
    } else if (!strncmp(in+i, "%year%", 6)) {
      strcat(obuff+j, year);
      i += 5;
      j += strlen(year);
    } else if (!strncmp(in+i, "%bitrate%", 9)) {
      strcat(obuff+j, bitrate);
      i += 8;
      j += strlen(bitrate);
    } else if (!strncmp(in+i, "%duration%", 10)) {
      strcat(obuff+j, duration);
      i += 9;
      j += strlen(duration);
    } else if (!strncmp(in+i, "%track%", 7)) {
      if (track>0)
	sprintf(buff, "%2d", track);
      else
	buff[0] = 0;
      strcat(obuff+j, buff);
      i += 6;
      j += strlen(buff);
    } else
      obuff[j++] = in[i];
  }
  obuff[j]=0;
  strncpy(out, obuff, len);
  out[len-1] = 0;
}

static void CenterDialog( HWND hwndDlg )
{
  HWND hwndOwner; 
  RECT rc, rcDlg, rcOwner; 

  hwndOwner = GetParent(hwndDlg);
  if (hwndOwner == NULL) hwndOwner = GetDesktopWindow(); 
  GetWindowRect(hwndOwner, &rcOwner); 
  GetWindowRect(hwndDlg, &rcDlg); 
  CopyRect(&rc, &rcOwner); 
  OffsetRect(&rcDlg, -rcDlg.left, -rcDlg.top); 
  OffsetRect(&rc, -rc.left, -rc.top); 
  OffsetRect(&rc, -rcDlg.right, -rcDlg.bottom); 
  SetWindowPos(hwndDlg, HWND_TOP, rcOwner.left + (rc.right / 2),
	       rcOwner.top + (rc.bottom / 2), 0, 0, SWP_NOSIZE); 
}

static void PositionWindow( HWND hdlg, POINT *pt)
{
  HDC hdc;
  if (pt->x!=0 && pt->y!=0) {
    hdc=CreateIC("DISPLAY", "", "", NULL);
    pt->x = min( GetDeviceCaps(hdc, HORZRES)-100, pt->x);
    pt->x = max( pt->x, 0 );
    pt->y = min( GetDeviceCaps(hdc, VERTRES)-100, pt->y);
    pt->y = max( pt->y, 0 );
    DeleteDC(hdc);
    SetWindowPos(hdlg, HWND_TOP, pt->x, pt->y, 0,0, SWP_NOSIZE);
  } else
    CenterDialog(hdlg);
}

static void SaveWindowPosition( HWND hdlg, POINT *pt )
{
  RECT saverect;
  GetWindowRect(hdlg, &saverect);
  pt->x = saverect.left;
  pt->y = saverect.top;
}

EXPORT int FileFilterInitialize( MINIFILEPARAM *param )
{
  DWORD type, len;
  static HINSTANCE WMADLL=NULL;

  // First, check that we can grok this plugin version!
  if (!param) return FALSE;
  if (param->version != PLUGINVERSION) return FALSE;

  // Load the WMCVCORE we need
  WMADLL = LoadLibrary("WMVCORE.DLL");
  if (!WMADLL) return FALSE;
  myWMCreateSyncReader = (HRESULT (__stdcall *)(IUnknown *,DWORD,IWMSyncReader ** ))GetProcAddress( WMADLL, "WMCreateSyncReader" );
  if (!myWMCreateSyncReader) return FALSE;

  strncpy(param->title, "WMA File Renamer", MAXTITLE);
  param->version = PLUGINVERSION;
  param->user = calloc(512, 1);
  strcpy((char*)param->user, "%title% - %artist%");
  strcpy((char*)param->user+256, "%duration%");

  len = 256;
  type = REG_SZ;
  RegQueryValueEx(param->userKey, "string", 0, &type,
		  (unsigned char *)param->user, &len);

  len = 256;
  type = REG_SZ;
  RegQueryValueEx(param->userKey, "commstring", 0, &type,
		  (unsigned char *)param->user+256, &len);

  type = REG_BINARY;
  len = sizeof(POINT);
  RegQueryValueEx(param->userKey, "position", 0, &type,
		  (unsigned char *)&position, &len);

  type = REG_DWORD;
  len = sizeof(DWORD);
  RegQueryValueEx(param->userKey, "enabled", 0, &type,
		  (unsigned char *)&enabled, &len);
	
  CoInitialize(NULL);
  
  return TRUE;
}


static BOOL CALLBACK Dialog(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
  static MINIFILEPARAM *param;
  static int last = 0;
  char buff[256], outBuff[256];
  int en;

  switch(msg) {
  case WM_USER:
    en = (SendDlgItemMessage(hDlg, IDC_ENABLE, BM_GETCHECK, 0, 0)==BST_CHECKED)
      ?TRUE:FALSE;
    EnableWindow(GetDlgItem(hDlg, IDC_EDIT), en);
    EnableWindow(GetDlgItem(hDlg, IDC_EDITCOMM), en);
    EnableWindow(GetDlgItem(hDlg, IDC_TITLE), en);
    EnableWindow(GetDlgItem(hDlg, IDC_ALBUM), en);
    EnableWindow(GetDlgItem(hDlg, IDC_ARTIST), en);
    EnableWindow(GetDlgItem(hDlg, IDC_BITRATE), en);
    EnableWindow(GetDlgItem(hDlg, IDC_DURATION), en);
    EnableWindow(GetDlgItem(hDlg, IDC_YEAR), en);
    EnableWindow(GetDlgItem(hDlg, IDC_TRACK), en);
    EnableWindow(GetDlgItem(hDlg, IDC_STATIC1), en);
    EnableWindow(GetDlgItem(hDlg, IDC_STATIC2), en);
    EnableWindow(GetDlgItem(hDlg, IDC_SAMPLE), en);
    EnableWindow(GetDlgItem(hDlg, IDC_SAMPLECOMM), en);
    EnableWindow(GetDlgItem(hDlg, IDC_STATICN), en);
    EnableWindow(GetDlgItem(hDlg, IDC_STATICC), en);
    if (en) {
      GetDlgItemText(hDlg, IDC_EDIT, buff, 64);
      FormatString(buff, "Filename.wma", outBuff, 256, "Song Title",
		   "The Windoze Band", "Red Album", "2002", 3, "64Kbps", "3:36");
      SetDlgItemText(hDlg, IDC_SAMPLE, outBuff);

      GetDlgItemText(hDlg, IDC_EDITCOMM, buff, 64);
      FormatString(buff, "1.43MB", outBuff, 256, "Song Title", "The Windoze Band",
		   "Red Album", "2002", 3, "128Kbps", "3:36");
      SetDlgItemText(hDlg, IDC_SAMPLECOMM, outBuff);
    } else {
      SetDlgItemText(hDlg, IDC_SAMPLE, "Filename.wma");
      SetDlgItemText(hDlg, IDC_SAMPLECOMM, "1.43MB");
    }
    return TRUE;

  case WM_INITDIALOG:
    param = (MINIFILEPARAM*)lParam;
    PositionWindow(hDlg, &position);
    SendDlgItemMessage(hDlg, IDC_ENABLE, BM_SETCHECK, 
		       enabled?BST_CHECKED:BST_UNCHECKED, 0);
    SetDlgItemText(hDlg, IDC_EDIT, (char*)param->user);
    SetDlgItemText(hDlg, IDC_EDITCOMM, (char*)param->user+256);
    SetFocus(GetDlgItem(hDlg, enabled?IDC_EDIT:IDC_ENABLE));
    SendMessage(hDlg, WM_USER, 0, 0);
    last = IDC_EDIT;
    return FALSE;


  case WM_COMMAND:
    switch (LOWORD(wParam)) {
    case IDOK:
      SaveWindowPosition(hDlg, &position);
      GetDlgItemText(hDlg, IDC_EDIT, (char*)param->user, 256);
      GetDlgItemText(hDlg, IDC_EDITCOMM, ((char*)param->user)+256, 256);
      if (SendDlgItemMessage(hDlg, IDC_ENABLE, BM_GETCHECK, 0, 0)==BST_CHECKED)
	enabled = TRUE;
      else
	enabled = FALSE;
      EndDialog(hDlg, TRUE);
      return TRUE;
		
    case IDCANCEL:
      SaveWindowPosition(hDlg, &position);
      EndDialog(hDlg, FALSE);
      return TRUE;
		
    case IDC_EDIT:
      last = IDC_EDIT;
      return SendMessage(hDlg, WM_USER, 0, 0);

    case IDC_EDITCOMM:
      last = IDC_EDITCOMM;
      return SendMessage(hDlg, WM_USER, 0, 0);

    case IDC_ENABLE:
      SendMessage(hDlg, WM_USER, 0, 0);
      return TRUE;

    case IDC_TITLE:
      if (last!=IDC_EDITCOMM) SendDlgItemMessage(hDlg, last, EM_REPLACESEL,
						 FALSE, (long)"%title%");
      SetFocus(GetDlgItem(hDlg, last));
      return TRUE;

    case IDC_ARTIST:
      if (last!=IDC_EDITCOMM) SendDlgItemMessage(hDlg, last, EM_REPLACESEL,
						 FALSE, (long)"%artist%");
      SetFocus(GetDlgItem(hDlg, last));
      return TRUE;

    case IDC_ALBUM:
      if (last!=IDC_EDITCOMM) SendDlgItemMessage(hDlg, last, EM_REPLACESEL,
						 FALSE, (long)"%album%");
      SetFocus(GetDlgItem(hDlg, last));
      return TRUE;

    case IDC_YEAR:
      if (last!=IDC_EDITCOMM) SendDlgItemMessage(hDlg, last, EM_REPLACESEL,
						 FALSE, (long)"%year%");
      SetFocus(GetDlgItem(hDlg, last));
      return TRUE;

    case IDC_TRACK:
      if (last!=IDC_EDITCOMM) SendDlgItemMessage(hDlg, last, EM_REPLACESEL,
						 FALSE, (long)"%track%");
      SetFocus(GetDlgItem(hDlg, last));
      return TRUE;

    case IDC_BITRATE:
      SendDlgItemMessage(hDlg, last, EM_REPLACESEL, FALSE, (long)"%bitrate%");
      SetFocus(GetDlgItem(hDlg, last));
      return TRUE;

    case IDC_DURATION:
      SendDlgItemMessage(hDlg, last, EM_REPLACESEL, FALSE, (long)"%duration%");
      SetFocus(GetDlgItem(hDlg, last));
      return TRUE;

    }
    break;
  }
  return FALSE;
}



EXPORT void FileFilterUserInterface( MINIFILEPARAM *param, HWND parent )
{
  DialogBoxParam(hInstance, MAKEINTRESOURCE(IDD_DIALOG), parent, 
		 Dialog, (LPARAM)param);
}

static int CheckWMA(char *fname, char *title, char *artist, char *album,
		      char *year, int *trackI, char *bitrate, char *duration)
{
  IWMSyncReader *pReader = 0;    // Pointer to the reader object.
  IWMHeaderInfo *pInfo = 0;   // Interface needed to read metadata.
  HRESULT hr;
  char ascii[8192];
  BSTR bstr;
  BYTE pAttr[8192];
  char track[64], author[64];
  WCHAR fnameW[8192];
  const WCHAR *name[]={g_wszWMTitle, g_wszWMAlbumArtist, g_wszWMAlbumTitle,
		       g_wszWMYear, g_wszWMTrackNumber, g_wszWMBitrate,
		       g_wszWMDuration, g_wszWMAuthor, NULL};
  char *value[]={title,       artist,             album,                          
		 year,              track,        bitrate,       
		 duration,   author,       NULL};

  
  hr = myWMCreateSyncReader(NULL, 0, &pReader);
  if (!SUCCEEDED(hr)) return FALSE;

  MultiByteToWideChar(GetACP(), 0, fname, -1, fnameW, 8192);

  bstr = SysAllocString(fnameW);
  hr = pReader->Open(bstr);
  SysFreeString(bstr);

  if (!SUCCEEDED(hr)) {
    pReader->Release();
    return FALSE;
  }

  // Query the reader for IWMHeaderInfo.
  hr = pReader->QueryInterface(IID_IWMHeaderInfo, reinterpret_cast<void**>(&pInfo));
  
  if (!SUCCEEDED(hr)) {
    pReader->Release();
    return FALSE;
  }

  for (int i=0; name[i]; i++) {
    // Find the data type and required buffer size.
    WMT_ATTR_DATATYPE Type;  // Receives the type.
    WORD cbLen = 8192;          // Receives the buffer size.
    WORD iStream = 0;        // Specifies the stream number.
    
    value[i][0] = 0;
    hr = pInfo->GetAttributeByName(&iStream, name[i], &Type, pAttr, &cbLen);
    if (SUCCEEDED(hr) && Type==WMT_TYPE_STRING && cbLen>0) {
      WideCharToMultiByte(GetACP(), 0, reinterpret_cast<LPWSTR>(pAttr), -1, ascii, 8192, NULL, NULL);
      strncpy(value[i], ascii, 64); value[i][63]=0;
    } else if (SUCCEEDED(hr) && Type==WMT_TYPE_DWORD && cbLen>0) {
      sprintf(ascii, "%d", *(unsigned int *)pAttr);
      strncpy(value[i], ascii, 64); value[i][63]=0;
    } else if (SUCCEEDED(hr) && Type==WMT_TYPE_QWORD && cbLen>0) {
      QWORD fatty;
      long sec;
      fatty = *(QWORD*)pAttr;
      fatty /= 10000000; // make into seconds
      sec = (long)fatty;
      sprintf(ascii, "%0d:%02d", sec/60, sec%60);
      strncpy(value[i], ascii, 64); value[i][63]=0;
    }
  }
  if (!artist[0]) strcpy(artist, author);
  if (bitrate[0]) {
    int br;
    sscanf( bitrate, "%d", &br );
    br = br + 1023;
    br /= 1024;
    sprintf(bitrate, "%dKbps", br);
  }
  if (track[0]) {
	sscanf(track, "%d", trackI);
  }
  pInfo->Release();
  pReader->Close();
  pReader->Release();

  return TRUE;
}	



EXPORT MINIZIPINFO *FileFilterExecute( MINIFILEPARAM *param,
				       MINIZIPINFO *mz,
				       char *label,
				       unsigned int labelSize,
				       char *path )
{
  int track;
  unsigned int i;
  char fname[MAX_PATH], buff[1024], title[128], artist[128];
  char album[128], year[128], bitrate[128], duration[128];
  char outBuff[1024];
  char *fileMap, *commMap, *dot;

  if (!enabled)
    return mz;

  fileMap = (char *)param->user;
  commMap = fileMap + 256;

  for (i=0; i<mz->numfiles; i++) {
		dot = strrchr(mz->fi[i].name, '.');
		if (!dot) dot = "x";
    if (!stricmp(dot, ".wma")) {
	  if (path[strlen(path)-1]=='\\')
	      sprintf(fname, "%s%s", path, mz->fi[i].name);
	  else
		sprintf(fname, "%s\\%s", path, mz->fi[i].name);
      buff[0] = 0;

      if (CheckWMA(fname, title, artist, album, year, &track, bitrate, duration)) {
		if (!title[0] || (title[0]==' '&&title[1]==0)) {
		  // No renaming...
		} else {
	      FormatString(fileMap, mz->fi[i].name, outBuff, 256,
		               title, artist, album, year, track,
		               bitrate, duration);
	      strncpy(mz->fi[i].name, outBuff, NAMELEN);
	      mz->fi[i].name[NAMELEN-1]=0;

		  FormatString(commMap, mz->fi[i].comm, outBuff, 256,
		  title, artist, album, year, track,
		  bitrate, duration);
		  strncpy(mz->fi[i].comm, outBuff, COMMLEN);
		  mz->fi[i].comm[COMMLEN-1]=0;
		}
      }    
    }

    if (mz->fi[i].zi) {
      sprintf(fname, "%s\\%s", path, mz->fi[i].name);
      FileFilterExecute( param, mz->fi[i].zi, label, labelSize, fname);
    }
  }
  return mz;
}

EXPORT void FileFilterUninitialize( MINIFILEPARAM *param )
{
  RegSetValueEx(param->userKey, "string", 0, REG_SZ,
		(unsigned char*)param->user, 256);
  RegSetValueEx(param->userKey, "commstring", 0, REG_SZ,
		(unsigned char*)param->user+256, 256);
  RegSetValueEx(param->userKey, "position", 0, REG_BINARY,
		(unsigned char*)&position, sizeof(POINT));
  RegSetValueEx(param->userKey, "enabled", 0, REG_DWORD,
		(unsigned char*)&enabled, sizeof(DWORD));
  free(param->user);
}


